@@ -27,11 +27,11 @@
 call_graph_endpoint <- function(token, operation, ..., options=list(),
                                 api_version=getOption("azure_graph_api_version"))
 {
-    url <- httr::parse_url(decode_jwt(token$credentials$access_token)$payload$aud)
+    url <- find_resource_host(token)
     url$path <- construct_path(api_version, operation)
     url$query <- options
 
-    call_graph_url(token, httr::build_url(url), ...)
+    call_graph_url(token, url, ...)
 }
 
 #' @rdname call_graph
@@ -41,7 +41,7 @@ call_graph_url <- function(token, url, ..., body=NULL, encode="json",
                            http_status_handler=c("stop", "warn", "message", "pass"),
                            auto_refresh=TRUE)
 {
-    headers <- process_headers(token, auto_refresh)
+    headers <- process_headers(token, url, auto_refresh)
 
     # if content-type is json, serialize it manually to ensure proper handling of nulls
     if(encode == "json" && !is_empty(body))
@@ -58,7 +58,7 @@ call_graph_url <- function(token, url, ..., body=NULL, encode="json",
 }
 
 
-process_headers <- function(token, auto_refresh)
+process_headers <- function(token, host, auto_refresh)
 {
     # if token has expired, renew it
     if(auto_refresh && !token$validate())
@@ -68,7 +68,7 @@ process_headers <- function(token, auto_refresh)
     }
 
     creds <- token$credentials
-    host <- httr::parse_url(AzureAuth::decode_jwt(creds$access_token)$payload$aud)$hostname
+    host <- httr::parse_url(host)$hostname
     headers <- c(
         Host=host,
         Authorization=paste(creds$token_type, creds$access_token),
@@ -172,3 +172,26 @@ get_confirmation <- function(msg, default=TRUE)
     else utils::askYesNo(msg, default)
     isTRUE(ok)
 }
+
+
+find_resource_host <- function(token)
+{
+    if(is_azure_v2_token(token))
+    {
+        # search the vector of scopes for the actual resource URL
+        url <- list()
+        i <- 1
+        while(is.null(url$scheme) && i <= length(token$scope))
+        {
+            url <- httr::parse_url(token$scope[i])
+            i <- i + 1
+        }
+    }
+    else url <- httr::parse_url(token$resource) # v1 token is the easy case
+
+    if(is.null(url$scheme))
+        stop("Could not find Graph host URL", call.=FALSE)
+    url$path <- NULL
+    url
+}
+
